#CLAN
Name         Marauder Family~
Shortname    marauder~
Filename     marauder.cr~
Description  ~
Leader       Rook~
NumberOne    Nfinet~
NumberTwo    ~
PKills       2
PDeaths      0
MKills       458
MDeaths      0
Type         1
Members      0
Board        0
Storeroom    0
GuardOne     0
atwar	   ~
GuardTwo     0
PatrolOne    0
PatrolTwo    0
Acro         MF~
TrooperOne   0
TrooperTwo   0
Troops       0
Funds        8071150
Jail         0
End

#END
