#PLANET
Name         Nal Hutta~
Filename     Nalhutta.pl~
X            1134
Y            -16541
Z            1024
Sector       0
Type    	   0
PopSupport   99
Starsystem   Y'toub~
GovernedBy   The Empire~
Area         nalhutta.are~
End

#END
