#PLANET
Name         Thorne Asteroid~
Filename     Thorne.pl~
X            -10002
Y            210
Z            1980
Sector       0
Type    	   0
PopSupport   100
Starsystem   Byss~
GovernedBy   Neutral~
Area         silenceindustries.are~
End

#END
