#PLANET
Name         Yavin IV~
Filename     Yavin.pl~
X            4000
Y            4000
Z            4000
Sector       0
Type    	   0
PopSupport   100
Starsystem   Yavin IV~
GovernedBy   Neutral~
Area         Yavin.are~
End

#END
