#HEADER

Name           XM-1 Missile Boat~
Sname          XM-1~
NumRooms       1
Cost           165150
Class          2
Tractor        0
primaryType    1
primaryCount   1
secondaryType  7
secondaryCount 0
RangeWeapons0  40
RangeWeapons1  20
RangeWeapons2  0
Hull           25
Shields        60
Energy         5000
Chaff          2
Maxbombs       0
Speed          55
Hyperspeed     2
Manuever       61
Turrets        0
Mods	    4
Clan           The Empire~
End

#ROOMS

#ROOM

RoomNum           1
Name              Cockpit of a Missile Boat~
Desc              The Alpha-class XM-1 Missileboat is a huge upgrade to the XG-1 Gunboat, armed with four times the payload. One of a few ships in the Imperial Fleet to possess a massive destructive capability exceeding that of a TIE Bomber. Along with the warhead launchers, it comes equipped with a single laser.~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          1
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End


#ENDROOMS

#END
