#HEADER

Name           Y-Wing Bomber~
Sname          Y-Wing~
NumRooms       1
Cost           73200
Class          2
Tractor        0
primaryType    2
primaryCount   1
secondaryType  7
secondaryCount 2
RangeWeapons0  24
RangeWeapons1  12
RangeWeapons2  0
Hull           120
Shields        33
Energy         3000
Chaff          0
Maxbombs       20
Speed          80
Hyperspeed     2
Manuever       59
Turrets        0
Mods	    2
Clan           The New Republic~
End

#ROOMS

#ROOM

RoomNum           1
Name              Cockpit of a Y-Wing Bomber~
Desc              Y-Wing is the common name of the Koensayr BTL Longprobe starfighter. Originally designed for long-range reconnaissance, this J-type snubfighter has twin propulsion systems behind a single cockpit. It is a two-man fighter, and is equipped with two laser and ion cannons, and has both missile and torpedo launchers.~
Flags             indoors spacecraft nomob ~
Tunnel            1
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          1
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End


#ENDROOMS

#END
