#PLANET
Name         Ord Mantell~
Filename     Ordmantell.pl~
X            -2800
Y            -1200
Z            3200
Sector       0
Type    	   0
PopSupport   -100
Starsystem   Ord Mantell~
GovernedBy   Neutral~
Area         OrdMantell.are~
Area         guild.are~
End

#END
