#PLANET
Name         Tatooine~
Filename     Tatooine.pl~
X            -1460
Y            -19001
Z            -1890
Sector       0
Type    	   0
PopSupport   92
Starsystem   Outer Rim~
GovernedBy   The Empire~
Area         tatooine.are~
Area         sith.are~
End

#END
