#PLANET
Name         Corellia~
Filename     Corellia.pl~
X            1000
Y            -1567
Z            26
Sector       0
Type    	   0
PopSupport   96
Starsystem   Corell~
GovernedBy   The Empire~
Area         Corellia.are~
End

#END
