#PLANET
Name         Kashyyyk~
Filename     Kashyyyk.pl~
X            1500
Y            -1397
Z            2230
Sector       0
Type    	   0
PopSupport   92
Starsystem   Kashyyyk~
GovernedBy   The Empire~
Area         kashyyyk.are~
End

#END
