#HEADER

Name           YT-1300 Transport Freighter~
Sname          YT-1300~
NumRooms       9
Cost           135950
Class          4
Tractor        0
primaryType    5
primaryCount   1
secondaryType  7
secondaryCount 0
RangeWeapons0  16
RangeWeapons1  0
RangeWeapons2  0
Hull           195
Shields        92
Energy         4200
Chaff          0
Maxbombs       0
Speed          88
Hyperspeed     1
Manuever       62
Turrets        2
Mods	    6
Clan           ~
End

#ROOMS

#ROOM

RoomNum           1
Name              Cockpit of a YT-1300 Freighter~
Desc              This is the cockpit of the YT-1300. Two seats are installed before a large console, which controls many aspects of the ship. The YT-1300 is a good medium-sized vessel, equipped with two quad-laser turrets, a shield generator, and an engine that boasts excellent speed for the YT's class. Southward leads to the lounge.~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       2
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          1
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           2
Name              The Lounge Area~
Desc              A typical recreation room, there are synthleather couches attached to the walls, and a table in the center. Two narrow corridors run up and down from here, leading to the turbolaser turrets. Ahead north is the cockpit, and south is the main entrance ramp.~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       1
Exits1       0
Exits2       3
Exits3       0
Exits4       4
Exits5       5
Exits6       0
Exits7       0
Exits8       6
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          22
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           3
Name              Entrance Ramp~
Desc              This wide corridor allows entrance and exit to the transport. A small camera mounted on what appears to be something resembling a laser turret guards the area leading into the ship. Northward you can see the lounge area, and further is the cockpit.~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       2
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          2
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           4
Name              Turbolaser Turret 1~
Desc              A small, cramped seat, this quad-laser turret is affixed to the top of the ship. The turbolaser itself is a Borstel RH8 Quadcannon, firing four beams almost simultaneously. A small corridor leads down into the main part of the ship.~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       2
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          12
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           5
Name              Turbolaser Turret 2~
Desc              A small, cramped seat, this quad-laser turret is affixed to the bottom of the ship. The turbolaser itself is a Borstel RH8 Quadcannon, firing four beams almost simultaneously. A small corridor leads up into the main part of the ship.~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       2
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          13
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           6
Name              A dim, cramped passageway~
Desc              You are below the lounge area, inside a small corridor that leads between the main part of the ship and the engineroom. Several exposed cables and piping run along the walls, and a few consoles are around displaying ship system information.~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       7
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       2
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          22
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           7
Name              The Engineroom~
Desc              Here is the internal engine access room of the YT-3000. Most system repair can be done from here, as the engine core is linked to all systems of the ship. A small passage is northward, leading to the rest of the ship.~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       6
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          7
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           8
Name              ~
Desc              ~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          0
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           9
Name              ~
Desc              ~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          0
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End


#ENDROOMS

#END
