#HEADER

Name           B-Wing Heavy Fighter~
Sname          B-Wing~
NumRooms       1
Cost           100100
Class          1
Tractor        0
primaryType    1
primaryCount   2
secondaryType  8
secondaryCount 1
RangeWeapons0  0
RangeWeapons1  16
RangeWeapons2  0
Hull           180
Shields        50
Energy         4000
Chaff          0
Maxbombs       0
Speed          91
Hyperspeed     2
Manuever       70
Turrets        0
Mods	    2
Clan           The New Republic~
End

#ROOMS

#ROOM

RoomNum           1
Name              Cockpit of a B-Wing Heavy fighter~
Desc              Designed by Admiral Ackbar and the Verpine colonies of Slayn and Korpil, the B-Wing looks like a flying cross, with the control pod at the top, retractable wings at the sides, and lasers at top and bottom. It is equipped with two lasers, two proton torpedo launchers, and three medium ion cannons.~
Flags             indoors spacecraft nomob ~
Tunnel            1
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          1
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End


#ENDROOMS

#END
