#PLANET
Name         Endor~
Filename     Endor.pl~
X            1300
Y            1100
Z            2300
Sector       0
Type    	   0
PopSupport   100
Starsystem   Endor~
GovernedBy   The Empire~
Area         endor.are~
End

#END
