#HEADER

Name           Toscan Q-8 Multirole Starfighter~
Sname          Q-8~
NumRooms       1
Cost           82450
Class          1
Tractor        0
primaryType    3
primaryCount   1
secondaryType  7
secondaryCount 2
RangeWeapons0  8
RangeWeapons1  0
RangeWeapons2  0
Hull           45
Shields        26
Energy         3500
Chaff          0
Maxbombs       0
Speed          87
Hyperspeed     2
Manuever       79
Turrets        0
Mods	    2
Clan           ~
End

#ROOMS

#ROOM

RoomNum           1
Name              Cockpit of a Toscan Q-8 Multirole Starfighter~
Desc              The Toscan Q-8 fighter was one of several designs that led to the development of the Y-Wing, hence why it uses a Koensayr cockpit. This two engine fighter was one of the few production fighters to be produced by Shobquix Yards. It has four laser and two ion cannons, and is also equipped with two missile launchers.~
Flags             indoors spacecraft nomob ~
Tunnel            1
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          1
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End


#ENDROOMS

#END
