#HEADER

Name           A-Wing Scout Fighter~
Sname          A-Wing~
NumRooms       1
Cost           96100
Class          1
Tractor        0
primaryType    1
primaryCount   2
secondaryType  7
secondaryCount 0
RangeWeapons0  12
RangeWeapons1  0
RangeWeapons2  0
Type           1
Hull           45
Shields        25
Energy         2500
Chaff          20
Maxbombs       0
Speed          120
Hyperspeed     1
Manuever       126
Turrets        0
Mods	    2
Clan           The New Republic~
End

#ROOMS

#ROOM

RoomNum           1
Name              Cockpit of an A-Wing~
Desc              The A-Wing is the generalized name of the Dodonna/Blissex RZ-1 starfighter. It is a flat, one-man, wedged-shaped snubfighter built for the Republic after the loss of the Hoth base and prior to the Battle of Endor. A unique feature of the A-Wing is the short hydro-servo bearing installed in each wingtip, allowing the pilot to tilt the laser cannons up to 60 degrees up or down. It boasts two laser cannons and has excellent speed and maneuverability because of its size.~
Flags             indoors spacecraft nomob ~
Tunnel            1
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          1
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End


#ENDROOMS

#END
