#PLANET
Name         Coruscant~
Filename     Coruscant.pl~
X            -345
Y            -987
Z            1562
Sector       0
Type    	   0
PopSupport   99
Starsystem   Coruscant~
GovernedBy   The Empire~
Area         coruscant.are~
Area         coruscant2.are~
Area         coruscant3.are~
Area         newbie.are~
End

#END
