#PLANET
Name         Mon Calamari~
Filename     Moncalamari.pl~
X            3300
Y            1300
Z            1700
Sector       0
Type    	   0
PopSupport   55
Starsystem   Calamari~
GovernedBy   The New Republic~
Area         calamari.are~
End

#END
