#HEADER

Name           T-4a Lambda Class Shuttle~
Sname          T-4a~
NumRooms       5
Cost           125300
Class          3
Tractor        0
primaryType    2
primaryCount   2
secondaryType  7
secondaryCount 0
RangeWeapons0  0
RangeWeapons1  0
RangeWeapons2  0
Hull           153
Shields        75
Energy         4975
Chaff          2
Maxbombs       0
Speed          65
Hyperspeed     3
Manuever       58
Turrets        0
Mods	    0
Clan           ~
End

#ROOMS

#ROOM

RoomNum           1
Name              Cockpit of a T-4a Lambda Class Shuttle~
Desc              The cockpit is large enough for a pilot copilot, and two passengers to sit in relative comfort.  The ships readouts show four lasers and a strong shield at the operator's disposal.~
Flags             indoors spacecraft nomob ~
Tunnel            4
Exits0       0
Exits1       0
Exits2       2
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          1
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           2
Name              T-4a Hallway~
Desc              The hall is large enough for two people to walk beside eachother.  Lights located at edges of the ceiling and floor illuminate the duraplast interior.~
Flags             indoors spacecraft nomob ~
Tunnel            2
Exits0       1
Exits1       0
Exits2       3
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          22
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           3
Name              T-4a Cargo Bay~
Desc              The T-4a's cargo bay takes up most of the ships 20 meters.  Parts of the wall fold out into benches and chairs.  There are also places on the floor where they can be installed.  The durasteel hatch is to the south.~
Flags             indoors spacecraft nomob ~
Tunnel            6
Exits0       2
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          2
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           4
Name              ~
Desc              ~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          0
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End

#ROOM

RoomNum           5
Name              ~
Desc              ~
Flags             indoors spacecraft nomob ~
Tunnel            315
Exits0       0
Exits1       0
Exits2       0
Exits3       0
Exits4       0
Exits5       0
Exits6       0
Exits7       0
Exits8       0
Exits9       0
Exflags0     0
Exflags1     0
Exflags2     0
Exflags3     0
Exflags4     0
Exflags5     0
Exflags6     0
Exflags7     0
Exflags8     0
Exflags9     0
Keys0        0
Keys1        0
Keys2        0
Keys3        0
Keys4        0
Keys5        0
Keys6        0
Keys7        0
Keys8        0
Keys9        0
RoomType          0
Rprog0        ~
Rprog1        ~
Rprog2        ~
Rprog3        ~
Rprog4        ~
Rprog5        ~
Rprog6        ~
Rprog7        ~
Rprog8        ~
Rprog9        ~
Reset0        ~
Reset1        ~
Reset2        ~
Reset3        ~
Reset4        ~
Reset5        ~
Reset6        ~
Reset7        ~
Reset8        ~
Reset9        ~
End


#ENDROOMS

#END
