#PLANET
Name         Bakura~
Filename     bakura.pl~
X            1001
Y            -520
Z            -1850
Sector       0
Type    	   0
PopSupport   98
Starsystem   Bakur~
GovernedBy   The Empire~
Area         bakura.are~
End

#END
